/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import _Object$defineProperties from '@babel/runtime-corejs2/core-js/object/define-properties';
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import cmp from '../../mixins/virtualCmp';
import { position, color, show } from '../../mixins/mixinProps';
import { makeColor } from '../../utils/cesiumHelpers';

var script = {
  name: 'vc-shine-ellipse',
  mixins: [cmp, position, color, show],
  props: {
    height: {
      type: Number,
      default: undefined
    },
    radius: {
      type: Number,
      default: 2000
    },
    deviationAlpha: {
      type: Number,
      default: 0.05
    },
    imageUrl: String
  },
  data: function data() {
    return {
      nowaiting: true,
      material: {}
    };
  },
  mounted: function mounted() {
    var _this = this;

    this.getParent(this.$parent).createPromise.then(function (_ref) {
      var Cesium = _ref.Cesium,
          viewer = _ref.viewer;
      var deviationAlpha = _this.deviationAlpha,
          color = _this.color,
          imageUrl = _this.imageUrl;
      var colorObject = makeColor(color);
      _this.flag = true;
      _this.x = 1;
      _this.material = {
        fabric: {
          type: 'Image',
          uniforms: {
            image: imageUrl,
            color: function color() {
              if (_this.flag) {
                _this.x -= deviationAlpha;
                _this.x <= 0 && (_this.flag = false);
              } else {
                _this.x += deviationAlpha;
                _this.x >= 1 && (_this.flag = true);
              }

              return colorObject.withAlpha(_this.x);
            }
          }
        }
      };
    });
  },
  methods: {
    createCesiumObject: function createCesiumObject() {
      var _this2 = this;

      return _regeneratorRuntime.async(function createCesiumObject$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt("return", this.$refs.entity.createPromise.then(function (_ref2) {
                var Cesium = _ref2.Cesium,
                    viewer = _ref2.viewer,
                    cesiumObject = _ref2.cesiumObject;

                if (!_this2.$refs.entity._mounted) {
                  return _this2.$refs.entity.load().then(function (_ref3) {
                    var Cesium = _ref3.Cesium,
                        viewer = _ref3.viewer,
                        cesiumObject = _ref3.cesiumObject;
                    return cesiumObject;
                  });
                } else {
                  return cesiumObject;
                }
              }));

            case 1:
            case "end":
              return _context.stop();
          }
        }
      }, null, this);
    },
    mount: function mount() {
      return _regeneratorRuntime.async(function mount$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              return _context2.abrupt("return", true);

            case 1:
            case "end":
              return _context2.stop();
          }
        }
      });
    },
    unmount: function unmount() {
      return _regeneratorRuntime.async(function unmount$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              return _context3.abrupt("return", this.$refs.entity && this.$refs.entity.unload());

            case 1:
            case "end":
              return _context3.stop();
          }
        }
      }, null, this);
    }
  },
  created: function created() {
    var _this3 = this;

    _Object$defineProperties(this, {
      entity: {
        enumerable: true,
        get: function get() {
          return _this3.cesiumObject;
        }
      }
    });
  }
};

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('i', {
    class: _vm.$options.name,
    staticStyle: {
      "display": "none !important"
    }
  }, [_c('vc-entity', {
    ref: "entity",
    attrs: {
      "position": _vm.position,
      "show": _vm.show
    }
  }, [_c('vc-graphics-ellipse', {
    attrs: {
      "height": _vm.height,
      "material": _vm.material,
      "semiMajorAxis": _vm.radius,
      "semiMinorAxis": _vm.radius
    }
  })], 1)], 1);
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* functional template */

var __vue_is_functional_template__ = false;
/* component normalizer */

function __vue_normalize__(template, style, script, scope, functional, moduleIdentifier, createInjector, createInjectorSSR) {
  var component = (typeof script === 'function' ? script.options : script) || {}; // For security concerns, we use only base name in production mode.

  component.__file = "VcShineEllipse.vue";

  if (!component.render) {
    component.render = template.render;
    component.staticRenderFns = template.staticRenderFns;
    component._compiled = true;
    if (functional) component.functional = true;
  }

  component._scopeId = scope;

  return component;
}
/* style inject */

/* style inject SSR */


var VcShineEllipse = __vue_normalize__({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__);

function plugin(Vue) {

  if (plugin.installed) {
    return;
  }

  plugin.installed = true;
  Vue.component(VcShineEllipse.name, VcShineEllipse);
}

export default plugin;
export { VcShineEllipse, plugin as install };
