/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import _classCallCheck from '@babel/runtime-corejs2/helpers/esm/classCallCheck';
import _createClass from '@babel/runtime-corejs2/helpers/esm/createClass';
import _defineProperty from '@babel/runtime-corejs2/helpers/esm/defineProperty';

var BinaryReader =
/*#__PURE__*/
function () {
  /**
   * 基于DataView和TypeArray，实现文件流读取
   * @param {ArrayBuffer|Uint8Array} buffer
   */
  function BinaryReader(buffer) {
    _classCallCheck(this, BinaryReader);

    this.buffer = buffer;
    this.position = 0;

    if (buffer instanceof Uint8Array) {
      this.buffer = buffer.buffer;
      this.position = buffer.byteOffset;
    }

    this.dataView = new DataView(this.buffer);
    this.length = this.dataView.byteLength;
    this.byteLength = this.length;
    /**
    *如果待读取的数据是按小端字节序写入的话，则按小端字节序读取。
    *在js中，解析较大数组时，如果按小端字节序读取会快很多，
    *所以在数据预处理阶段可以考虑将输出结果按小端字节序写入数据文件
    */

    this.isLittleEndian = false;
  }
  /**
   * 跳过指定字节数
   * @param {Number} skipCount
   */


  _createClass(BinaryReader, [{
    key: "skip",
    value: function skip(skipCount) {
      var length = skipCount;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('范围超过了字节流末尾，请检查count参数');
      }

      this.position += length;
    }
    /**
     * 将该流的当前位置设置为给定值。并返回流中的新位置
     * @param {Number} offset 相对于 origin 的点，从此处开始查找。
     * @param {BinaryReader.SeekOrigin} origin 使用 BinaryReader.SeekOrigin 类型的值，将开始位置、结束位置或当前位置指定为 origin 的参考点。
     */

  }, {
    key: "seek",
    value: function seek(offset, origin) {
      if (typeof origin === 'undefined' || origin == null) {
        origin = BinaryReader.SeekOrigin.Begin;
      }

      switch (origin) {
        case BinaryReader.SeekOrigin.Begin:
          this.position = offset;
          break;

        case BinaryReader.SeekOrigin.Current:
          this.position += offset;
          break;

        case BinaryReader.SeekOrigin.End:
          this.position = this.dataView.byteLength - offset;
          break;

        default:
          this.position = offset;
          break;
      }

      return this.position;
    }
    /**
     * 从流中读取字节块并将该数据写入给定缓冲区中。并将读取位置提升count个字节
     * @param {Int8Array|Array.<Number>} array
     * @param {Number} offset array 中的字节偏移量
     * @param {Number} count 读取的字节数
     * @param {Boolean} utf8
     */

  }, {
    key: "read",
    value: function read(array, offset, count, utf8) {
      var length = count;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        return -1; // thrownew Error("范围超过了字节流末尾，请检查count参数");
      }

      var vals;

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);
        vals = utf8 ? new Uint8Array(buffer) : new Int8Array(buffer);
        this.position += length;

        for (var i = offset; i < offset + count; i++) {
          array[i] = vals[i];
        }
      } else {
        for (var _i = 0; _i < count; _i++) {
          array[_i + offset] = this.readByte();
        }
      }

      return length;
    }
    /**
     * 从流中读取一个byte数值，并将读取位置提升1个字节。如果已到达流的末尾，则返回 -1。
     * @param {Boolean} toUint8Array
     */

  }, {
    key: "readByte",
    value: function readByte(toUint8Array) {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val; //! this.isLittleEndian);

      if (!toUint8Array) {
        val = this.dataView.getInt8(this.position, true);
      } else {
        val = this.dataView.getUint8(this.position, true);
      }

      this.position += 1;
      return val;
    }
    /**
     * 从当前位置开始，读取指定长度(count)的byte数组,并将读取位置提升count个字节。如果已到达流的末尾，则返回 -1。
     * @param {Number} count 指定长度
     * @param {Boolean} toUint8Array 是否用Uint8存储byte数据，js中，Uint8范围是0~255，Int8是-128~128
     */

  }, {
    key: "readBytes",
    value: function readBytes(count, toUint8Array) {
      var length = count;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('范围超过了字节流末尾，请检查count参数');
      }

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);

        if (toUint8Array) {
          var _vals2 = new Uint8Array(buffer);

          this.position += length;
          return _vals2;
        }

        var _vals = new Int8Array(buffer);

        this.position += length;
        return _vals;
      } // var buffer = this.dataView.buffer.slice(this.position, this.position + length);


      if (toUint8Array) {
        var _vals3 = new Uint8Array(count);

        for (var i = 0; i < _vals3.length; i++) {
          _vals3[i] = this.readByte(toUint8Array);
        }

        return _vals3;
      }

      var vals = new Int8Array(count);

      for (var _i2 = 0; _i2 < vals.length; _i2++) {
        vals[_i2] = this.readByte();
      }

      return vals;
    }
    /**
     * 从流中读取一个byte数值，并将读取位置提升1个字节。如果已到达流的末尾，则返回 -1。
     */

  }, {
    key: "readUbyte",
    value: function readUbyte() {
      return this.readByte(true);
    }
    /**
     * 从流中读取一个short数值，并将读取位置提升2个字节。如果已到达流的末尾，则返回 -1。
     */

  }, {
    key: "readShort",
    value: function readShort() {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getInt16(this.position, this.isLittleEndian);
      this.position += 2;
      return val;
    }
    /**
     * 从当前位置开始，读取指定长度(count)的short数组,并将读取位置提升2*count个字节。如果已到达流的末尾，则返回 -1。
     * @param {Number} count
     */

  }, {
    key: "readShorts",
    value: function readShorts(count) {
      var length = count * Int16Array.BYTES_PER_ELEMENT;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('已到达流的末尾，请检查count参数');
      }

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);

        var _vals4 = new Int16Array(buffer);

        this.position += length;
        buffer = null;
        return _vals4;
      }

      var vals = new Int16Array(count);

      for (var i = 0; i < vals.length; i++) {
        vals[i] = this.readShort();
      }

      return vals;
    }
    /**
     * 从流中读取一个short数值，并将读取位置提升2个字节。如果已到达流的末尾，则返回 -1。
     */

  }, {
    key: "readUShort",
    value: function readUShort() {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getUint16(this.position, this.isLittleEndian);
      this.position += 2;
      return val;
    }
    /**
     * 从当前位置开始，读取指定长度(count)的short数组,并将读取位置提升2*count个字节。如果已到达流的末尾，则返回 -1。
     * @param {*} count
     */

  }, {
    key: "readUShorts",
    value: function readUShorts(count) {
      var length = count * Uint16Array.BYTES_PER_ELEMENT;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('已到达流的末尾，请检查count参数');
      }

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);

        var _vals5 = new Uint16Array(buffer);

        this.position += length;
        buffer = null;
        return _vals5;
      }

      var vals = new Uint16Array(count);

      for (var i = 0; i < vals.length; i++) {
        vals[i] = this.readUShort();
      }

      return vals;
    }
    /**
     * 从流中读取一个int数值，并将读取位置提升4个字节。如果已到达流的末尾，则为 -1。
     */

  }, {
    key: "readInt",
    value: function readInt() {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getInt32(this.position, this.isLittleEndian);
      this.position += 4;
      return val;
    }
    /**
     * 从当前位置开始，读取指定长度(count)的int数组,并将读取位置提升4*count个字节。如果已到达流的末尾，则返回 -1。
     * @param {*} count
     */

  }, {
    key: "readInts",
    value: function readInts(count) {
      var length = count * Int32Array.BYTES_PER_ELEMENT;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('已到达流的末尾，请检查count参数');
      }

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);

        var _vals6 = new Int32Array(buffer);

        this.position += length;
        buffer = null;
        return _vals6;
      }

      var vals = new Int32Array(count);

      for (var i = 0; i < vals.length; i++) {
        vals[i] = this.readInt();
      }

      return vals;
    }
    /**
     * 从流中读取一个Uint32数值，并将读取位置提升4个字节。如果已到达流的末尾，则为 -1。
     */

  }, {
    key: "readUint",
    value: function readUint() {
      if (this.position + 4 > this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getUint32(this.position, this.isLittleEndian);
      this.position += 4;
      return val;
    }
    /**
     * 从流中读取一个Int64数值，并将读取位置提升8个字节。如果已到达流的末尾，则为 -1。
     */

  }, {
    key: "readInt64",
    value: function readInt64() {
      if (!BinaryReader.BigInteger) {
        throw ReferenceError('引用的BinaryReader.BigInteger未定义');
      }

      if (this.position + 8 > this.dataView.byteLength) {
        return -1;
      }

      var bytes = this.readBytes(8, true);
      var arr = [];

      if (this.isLittleEndian) {
        for (var i = 0; i < bytes.length; i++) {
          arr[i] = bytes[bytes.length - 1 - i];
        }

        bytes = new Uint8Array(arr);
      }

      var long = bytes[0] & 0xFF;
      long += (bytes[1] & 0xFF) << 8;
      long += (bytes[2] & 0xFF) << 16;
      long += (bytes[3] & 0xFF) << 24;
      long += (bytes[4] & 0xFF) * Math.pow(2, 32);
      long += (bytes[5] & 0xFF) * Math.pow(2, 40);
      long += (bytes[6] & 0xFF) * Math.pow(2, 48);
      long += bytes[7] * Math.pow(2, 56);
      return long;
    }
    /**
     * 从流中读取一个Uint64数值，并将读取位置提升8个字节。如果已到达流的末尾，则为 -1。
     */

  }, {
    key: "readUint64",
    value: function readUint64() {
      if (!BinaryReader.BigInteger) {
        throw ReferenceError('引用的BinaryReader.BigInteger未定义');
      }

      if (this.position + 8 > this.dataView.byteLength) {
        return -1;
      }

      var bytes = this.readBytes(8, true);
      var arr = [];

      if (!this.isLittleEndian) {
        for (var i = 0; i < bytes.length; i++) {
          arr[i] = bytes[bytes.length - 1 - i];
        }

        bytes = new Uint8Array(arr);
      }

      var long = bytes[0] & 0xFF;
      long += (bytes[1] & 0xFF) << 8;
      long += (bytes[2] & 0xFF) << 16;
      long += (bytes[3] & 0xFF) << 24;
      long += (bytes[4] & 0xFF) * Math.pow(2, 32);
      long += (bytes[5] & 0xFF) * Math.pow(2, 40);
      long += (bytes[6] & 0xFF) * Math.pow(2, 48);
      long += bytes[7] * Math.pow(2, 56);
      return long;
    }
    /**
     * 从流中读取一个float数值，并将读取位置提升4个字节。如果已到达流的末尾，则为 -1。
     */

  }, {
    key: "readFloat",
    value: function readFloat() {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getFloat32(this.position, this.isLittleEndian);
      this.position += 4;
      return val;
    }
    /**
     * 从当前位置开始，读取指定长度(count)的Float数组,并将读取位置提升4*count个字节。如果已到达流的末尾，则返回 -1。
     * @param {*} count
     */

  }, {
    key: "readFloats",
    value: function readFloats(count) {
      var length = count * Float32Array.BYTES_PER_ELEMENT;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('已到达流的末尾，请检查count参数');
      }

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);

        var _vals7 = new Float32Array(buffer);

        this.position += length;
        buffer = null;
        return _vals7;
      }

      var vals = new Float32Array(count);

      for (var i = 0; i < vals.length; i++) {
        vals[i] = this.readFloat();
      }

      return vals;
    }
  }, {
    key: "readDouble",
    value: function readDouble() {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getFloat64(this.position, this.isLittleEndian);
      this.position += 8;
      return val;
    }
  }, {
    key: "readDoubles",
    value: function readDoubles(count) {
      var length = count * Float64Array.BYTES_PER_ELEMENT;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('已到达流的末尾，请检查count参数');
      }

      if (this.isLittleEndian) {
        var buffer = this.dataView.buffer.slice(this.position, this.position + length);

        var _vals8 = new Float64Array(buffer);

        this.position += length;
        buffer = null;
        return _vals8;
      }

      var vals = new Float64Array(count);

      for (var i = 0; i < vals.length; i++) {
        vals[i] = this.readDouble();
      }

      return vals;
    }
  }, {
    key: "readChar",
    value: function readChar() {
      if (this.position >= this.dataView.byteLength) {
        return -1;
      }

      var val = this.dataView.getUint8(this.position, true); //! this.isLittleEndian);
      // val = val >= 0 ? val : 256 + val;

      val = String.fromCharCode(val);
      this.position += 1;
      return val;
    }
  }, {
    key: "readChars",
    value: function readChars(count) {
      var length = count;

      if (this.position >= this.dataView.byteLength || this.position + length > this.dataView.byteLength) {
        throw new Error('范围超过了字节流末尾，请检查count参数');
      }

      var vals = '';

      for (var i = 0; i < count; i++) {
        vals += this.readChar();
      }

      return vals;
    }
  }, {
    key: "dispose",
    value: function dispose() {
      this.buffer = null;
      this.position = null;
      this.dataView = null;
    }
  }]);

  return BinaryReader;
}();

_defineProperty(BinaryReader, "SeekOrigin", {
  /**
  * 指定流的开头。
  */
  Begin: 0,

  /**
  * 指定流内的当前位置。
  */
  Current: 1,

  /**
  * 指定流的结尾。
  */
  End: 2
});

export default BinaryReader;
