/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import _Object$defineProperties from '@babel/runtime-corejs2/core-js/object/define-properties';
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import bindEvents from '../../utils/bindEvent';
import { Events } from '../../utils/events';
import cmp from '../virtualCmp';
import { show } from '../mixinProps';
import mergeDescriptors from '../../utils/mergeDescriptors';

var methods = {
  mount: function mount() {
    var _this = this;

    var dataSources, datasource;
    return _regeneratorRuntime.async(function mount$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            dataSources = this.dataSources, datasource = this.datasource;
            bindEvents.call(this, datasource, Events['datasource-events'], true);
            Events['datasource-property-events'].forEach(function (eventName) {
              datasource[eventName.name] && bindEvents.call(_this, datasource[eventName.name], eventName.events, true);
            });
            datasource.show = this.show;
            return _context.abrupt("return", dataSources && dataSources.add(datasource));

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, null, this);
  },
  unmount: function unmount() {
    var _this2 = this;

    var dataSources, datasource;
    return _regeneratorRuntime.async(function unmount$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            dataSources = this.dataSources, datasource = this.datasource;
            bindEvents.call(this, datasource, Events['datasource-events'], false);
            Events['datasource-property-events'].forEach(function (eventName) {
              datasource[eventName.name] && bindEvents.call(_this2, datasource[eventName.name], eventName.events, false);
            });
            return _context2.abrupt("return", dataSources && dataSources.remove(datasource));

          case 4:
          case "end":
            return _context2.stop();
        }
      }
    }, null, this);
  },
  getServices: function getServices() {
    var vm = this;
    return mergeDescriptors(cmp.methods.getServices.call(this), {
      get datasource() {
        return vm.datasource;
      },

      get entities() {
        return vm.datasource.entities;
      }

    });
  }
};
var mixinDatasource = {
  mixins: [cmp, show],
  methods: methods,
  stubVNode: {
    attrs: function attrs() {
      return {
        class: this.$options.name
      };
    }
  },
  created: function created() {
    var _this3 = this;

    _Object$defineProperties(this, {
      datasource: {
        enumerable: true,
        get: function get() {
          return _this3.cesiumObject;
        }
      },
      dataSources: {
        enumerable: true,
        get: function get() {
          return _this3.$services && _this3.$services.dataSources;
        }
      }
    });
  }
};

export default mixinDatasource;
