/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import { lnglatValidator } from '../utils/util';

/**
 * @const {Object}  position mixin
 */

var position = {
  props: {
    position: {
      type: Object | Function,
      validator: function validator(val) {
        return val && Object.prototype.hasOwnProperty.call(val, 'lng') ? lnglatValidator(val.lng, val.lat) : true;
      }
    }
  }
};
/**
 * @const {Boolean}  show mixin
 */

var show = {
  props: {
    show: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Object}  distanceDisplayCondition mixin
 */

var distanceDisplayCondition = {
  props: {
    distanceDisplayCondition: Object
  }
};
/**
 * @const {String | Object} image mixin
 */

var image = {
  props: {
    image: String | Object
  }
};
/**
 * @const {Number} scale mixin
 */

var scale = {
  props: {
    scale: {
      type: Number,
      default: 1.0
    }
  }
};
/**
 * @const {Object}  pixelOffset mixin
 */

var pixelOffset = {
  props: {
    pixelOffset: {
      type: Object,
      default: function _default() {
        return {
          x: 0,
          y: 0
        };
      }
    }
  }
};
/**
 * @const {Object}  eyeOffset mixin
 */

var eyeOffset = {
  props: {
    eyeOffset: {
      type: Object,
      default: function _default() {
        return {
          x: 0,
          y: 0,
          z: 0
        };
      }
    }
  }
};
/**
 * @const {Number} horizontalOrigin mixin
 */

var horizontalOrigin = {
  props: {
    horizontalOrigin: {
      type: Number,
      default: 0
    }
  }
};
/**
 * @const {Number}  verticalOrigin mixin
 */

var verticalOrigin = {
  props: {
    verticalOrigin: {
      type: Number,
      default: 0
    }
  }
};
/**
 * @const {Number} heightReference mixin
 */

var heightReference = {
  props: {
    heightReference: Number
  }
};
/**
 * @const {Object|String|Array} color mixin
 */

var color = {
  props: {
    color: {
      type: Object | String | Array,
      default: 'white'
    }
  }
};
/**
 * @const {Number|Object} rotation mixin
 */

var rotation = {
  props: {
    rotation: {
      type: Number | Object,
      default: 0
    }
  }
};
/**
 * @const {Object} alignedAxis mixin
 */

var alignedAxis = {
  props: {
    alignedAxis: {
      type: Object,
      default: function _default() {
        return {
          x: 0,
          y: 0,
          z: 0
        };
      }
    }
  }
};
/**
 * @const {Boolean} sizeInMeters mixin
 */

var sizeInMeters = {
  props: {
    sizeInMeters: Boolean
  }
};
/**
 * @const {Number} width mixin
 */

var width = {
  props: {
    width: Number
  }
};
/**
 * @const {Number} height mixin
 */

var height = {
  props: {
    height: Number
  }
};
/**
 * @const {Object} scaleByDistance mixin
 */

var scaleByDistance = {
  props: {
    scaleByDistance: Object
  }
};
/**
 * @const {Object} translucencyByDistance mixin
 */

var translucencyByDistance = {
  props: {
    translucencyByDistance: Object
  }
};
/**
 * @const {Object} pixelOffsetScaleByDistance mixin
 */

var pixelOffsetScaleByDistance = {
  props: {
    pixelOffsetScaleByDistance: Object
  }
};
/**
 * @const {Number} disableDepthTestDistance mixin
 */

var disableDepthTestDistance = {
  props: {
    disableDepthTestDistance: Number
  }
};
/**
 * @const {Object} dimensions mixin
 */

var dimensions = {
  props: {
    dimensions: Object | Function
  }
};
/**
 * @const {Boolean} fill mixin
 */

var fill = {
  props: {
    fill: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Object|String|Array} material mixin
 */

var material = {
  props: {
    material: {
      type: Object | String | Array,
      default: 'white'
    }
  }
};
/**
 * @const {Boolean} outline mixin
 */

var outline = {
  props: {
    outline: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const {Object|String|Array} outlineColor mixin
 */

var outlineColor = {
  props: {
    outlineColor: {
      type: Object | String | Array,
      default: 'black'
    }
  }
};
/**
 * @const {Number} outlineWidth mixin
 */

var outlineWidth = {
  props: {
    outlineWidth: {
      type: Number,
      default: 1.0
    }
  }
};
/**
 * @const {Number} shadows mixin
 */

var shadows = {
  props: {
    shadows: Number
  }
};
/**
 * @const {Array} positions mixin
 */

var positions = {
  props: {
    positions: Array | Object
  }
};
/**
 * @const {Number} extrudedHeight mixin
 */

var extrudedHeight = {
  props: {
    extrudedHeight: Number
  }
};
/**
 * @const {Number} extrudedHeightReference mixin
 */

var extrudedHeightReference = {
  props: {
    extrudedHeightReference: Number
  }
};
/**
 * @const {Number} cornerType mixin
 */

var cornerType = {
  props: {
    cornerType: {
      type: Number,
      default: 0
    }
  }
};
/**
 * @const {Number} granularity mixin
 */

var granularity = {
  props: {
    granularity: Number
  }
};
/**
 * @const {Number} classificationType mixin
 */

var classificationType = {
  props: {
    classificationType: {
      type: Number,
      default: 2
    }
  }
};
/**
 * @const {Number} zIndex mixin
 */

var zIndex = {
  props: {
    zIndex: Number
  }
};
/**
 * @const {Number} length mixin
 */

var length = {
  props: {
    length: Number
  }
};
/**
 * @const {Number} topRadius mixin
 */

var topRadius = {
  props: {
    topRadius: Number
  }
};
/**
 * @const {Number} bottomRadius mixin
 */

var bottomRadius = {
  props: {
    bottomRadius: Number
  }
};
/**
 * @const {Number} numberOfVerticalLines mixin
 */

var numberOfVerticalLines = {
  props: {
    numberOfVerticalLines: {
      type: Number,
      default: 16
    }
  }
};
/**
 * @const {Number} slices mixin
 */

var slices = {
  props: {
    slices: {
      type: Number,
      default: 128
    }
  }
};
/**
 * @const {Number} semiMajorAxis mixin
 */

var semiMajorAxis = {
  props: {
    semiMajorAxis: [Number, Object]
  }
};
/**
 * @const {Number} semiMinorAxis mixin
 */

var semiMinorAxis = {
  props: {
    semiMinorAxis: [Number, Object]
  }
};
/**
 * @const {Number|Object} stRotation mixin
 */

var stRotation = {
  props: {
    stRotation: {
      type: [Number, Object],
      default: 0.0
    }
  }
};
/**
 * @const {Number} radii mixin
 */

var radii = {
  props: {
    radii: Object
  }
};
/**
 * @const {Number} stackPartitions mixin
 */

var stackPartitions = {
  props: {
    stackPartitions: {
      type: Number,
      default: 64
    }
  }
};
/**
 * @const {Number} slicePartitions mixin
 */

var slicePartitions = {
  props: {
    slicePartitions: {
      type: Number,
      default: 64
    }
  }
};
/**
 * @const {Number} subdivisions mixin
 */

var subdivisions = {
  props: {
    subdivisions: {
      type: Number,
      default: 128
    }
  }
};
/**
 * @const {String} text mixin
 */

var text = {
  props: {
    text: String
  }
};
/**
 * @const {String} font mixin
 */

var font = {
  props: {
    font: {
      type: String,
      default: '30px sans-serif'
    }
  }
};
/**
 * @const {Number} labelStyle mixin
 */

var labelStyle = {
  props: {
    labelStyle: {
      type: Number,
      default: 0
    }
  }
};
/**
 * @const {Boolean} showBackground mixin
 */

var showBackground = {
  props: {
    showBackground: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const {Object/String/Array} backgroundColor mixin
 */

var backgroundColor = {
  props: {
    backgroundColor: {
      type: Object | String | Array,
      default: function _default() {
        return [0.165, 0.165, 0.165, 0.8];
      }
    }
  }
};
/**
 * @const {Object} backgroundPadding mixin
 */

var backgroundPadding = {
  props: {
    backgroundPadding: {
      type: Object,
      default: function _default() {
        return {
          x: 7,
          y: 5
        };
      }
    }
  }
};
/**
 * @const {Object|String|Array} fillColor mixin
 */

var fillColor = {
  props: {
    fillColor: {
      type: Object | String | Array,
      default: 'WHITE'
    }
  }
};
/**
 * @const {String} uri mixin
 */

var uri = {
  props: {
    uri: String
  }
};
/**
 * @const {Number} minimumPixelSize mixin
 */

var minimumPixelSize = {
  props: {
    minimumPixelSize: {
      type: Number,
      default: 0.0
    }
  }
};
/**
 * @const {Number} maximumScale mixin
 */

var maximumScale = {
  props: {
    maximumScale: Number
  }
};
/**
 * @const {Boolean} incrementallyLoadTextures mixin
 */

var incrementallyLoadTextures = {
  props: {
    incrementallyLoadTextures: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Boolean} clampAnimations mixin
 */

var clampAnimations = {
  props: {
    clampAnimations: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Object|String|Array} silhouetteColor mixin
 */

var silhouetteColor = {
  props: {
    silhouetteColor: Object | String | Array
  }
};
/**
 * @const {Number} silhouetteSize mixin
 */

var silhouetteSize = {
  props: {
    silhouetteSize: {
      type: Number,
      default: 0.0
    }
  }
};
/**
 * @const {Number} colorBlendMode mixin
 */

var colorBlendMode = {
  props: {
    colorBlendMode: {
      type: Number,
      default: 0
    }
  }
};
/**
 * @const {Number} colorBlendAmount mixin
 */

var colorBlendAmount = {
  props: {
    colorBlendAmount: {
      type: Number,
      default: 0.5
    }
  }
};
/**
 * @const {Object} clippingPlanes mixin
 */

var clippingPlanes = {
  props: {
    clippingPlanes: Object
  }
};
/**
 * @const {Number} pixelSize mixin
 */

var pixelSize = {
  props: {
    pixelSize: {
      type: Number,
      default: 1
    }
  }
};
/**
 * @const {Boolean} perPositionHeight mixin
 */

var perPositionHeight = {
  props: {
    perPositionHeight: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const {Boolean} closeTop mixin
 */

var closeTop = {
  props: {
    closeTop: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Boolean} closeBottom mixin
 */

var closeBottom = {
  props: {
    closeBottom: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Number} arcType mixin
 */

var arcType = {
  props: {
    arcType: {
      type: Number,
      default: 1
    }
  }
};
/**
 * @const {Boolean} clampToGround mixin
 */

var clampToGround = {
  props: {
    clampToGround: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const Array{} minimumHeights mixin
 */

var minimumHeights = {
  props: {
    minimumHeights: Array
  }
};
/**
 * @const {Array} maximumHeights mixin
 */

var maximumHeights = {
  props: {
    maximumHeights: Array
  }
}; // imageryProvider

/**
 * @const {String|Object} url mixin
 */

var url = {
  props: {
    url: String | Object
  }
};
/**
 * @const {String} token mixin
 */

var token = {
  props: {
    token: String
  }
};
/**
 * @const {Object} tileDiscardPolicy mixin
 */

var tileDiscardPolicy = {
  props: {
    tileDiscardPolicy: Object
  }
};
/**
 * @const {String} layers mixin
 */

var layers = {
  props: {
    layers: String
  }
};
/**
 * @const {Boolean} enablePickFeatures mixin
 */

var enablePickFeatures = {
  props: {
    enablePickFeatures: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Object} rectangle mixin
 */

var rectangle = {
  props: {
    rectangle: Object
  }
};
/**
 * @const {Object} tilingScheme mixin
 */

var tilingScheme = {
  props: {
    tilingScheme: Object
  }
};
/**
 * @const {Object} ellipsoid mixin
 */

var ellipsoid = {
  props: {
    ellipsoid: Object
  }
};
/**
 * @const {String|Object} credit mixin
 */

var credit = {
  props: {
    credit: {
      type: String | Object,
      default: ''
    }
  }
};
/**
 * @const {Number} tileWidth mixin
 */

var tileWidth = {
  props: {
    tileWidth: {
      type: Number,
      default: 256
    }
  }
};
/**
 * @const {Number} tileHeight mixin
 */

var tileHeight = {
  props: {
    tileHeight: {
      type: Number,
      default: 256
    }
  }
};
/**
 * @const {Number} maximumLevel mixin
 */

var maximumLevel = {
  props: {
    maximumLevel: Number
  }
};
/**
 * @const {Number} minimumLevel mixin
 */

var minimumLevel = {
  props: {
    minimumLevel: {
      type: Number,
      default: 0
    }
  }
};
/**
 * @const {String} fileExtension mixin
 */

var fileExtension = {
  props: {
    fileExtension: {
      type: String,
      default: 'png'
    }
  }
};
/**
 * @const {String} accessToken mixin
 */

var accessToken = {
  props: {
    accessToken: String
  }
};
/**
 * @const {String} format mixin
 */

var format = {
  props: {
    format: {
      type: String,
      default: 'png'
    }
  }
};
/**
 * @const {String|Array} subdomains mixin
 */

var subdomains = {
  props: {
    subdomains: String | Array
  }
};
/**
 * @const {Array} getFeatureInfoFormats mixin
 */

var getFeatureInfoFormats = {
  props: {
    getFeatureInfoFormats: Array
  }
};
/**
 * @const {Object} clock mixin
 */

var clock = {
  props: {
    clock: Object
  }
};
/**
 * @const {Object} times mixin
 */

var times = {
  props: {
    times: Object
  }
}; // primitive 相关

/**
 * @const {Boolean}
 * allowPicking
 * asynchronous
 */

var aaMixin = {
  props: {
    allowPicking: {
      type: Boolean,
      default: true
    },
    asynchronous: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Boolean} debugShowShadowVolume mixin
 */

var debugShowShadowVolume = {
  props: {
    debugShowShadowVolume: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const {Boolean} releaseGeometryInstances mixin
 */

var releaseGeometryInstances = {
  props: {
    releaseGeometryInstances: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Boolean} interleave mixin
 */

var interleave = {
  props: {
    interleave: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const {Object} appearance mixin
 */

var appearance = {
  props: {
    appearance: Object
  }
};
/**
 * @const {Array|Object} geometryInstances mixin
 */

var geometryInstances = {
  props: {
    geometryInstances: Array | Object
  }
};
/**
 * @const {Boolean}
 * vertexCacheOptimize
 * compressVertices
 */

var vcMixin = {
  props: {
    vertexCacheOptimize: {
      type: Boolean,
      default: false
    },
    compressVertices: {
      type: Boolean,
      default: true
    }
  }
};
/**
 * @const {Object} modelMatrix mixin
 */

var modelMatrix = {
  props: {
    modelMatrix: Object
  }
};
/**
 * @const {Boolean} debugShowBoundingVolume mixin
 */

var debugShowBoundingVolume = {
  props: {
    debugShowBoundingVolume: {
      tyep: Boolean,
      default: false
    }
  }
};
/**
 * @const {Object} scene mixin
 */

var scene = {
  props: {
    scene: Object
  }
};
/**
 * @const {Number} blendOption mixin
 */

var blendOption = {
  props: {
    blendOption: {
      type: Number,
      default: 2
    }
  }
};
/**
 * @const {*} id mixin
 */

var id = {
  props: {
    id: null
  }
};
/**
 * @const {Boolean} loop mixin
 */

var loop = {
  props: {
    loop: {
      type: Boolean,
      default: false
    }
  }
};
/**
 * @const {Boolean} debugWireframe mixin
 */

var debugWireframe = {
  props: {
    debugWireframe: {
      type: Boolean,
      default: false
    }
  }
}; // geometry 相关

/**
 * @const {Object} vertexFormat mixin
 */

var vertexFormat = {
  props: {
    vertexFormat: Object
  }
};
/**
 * @const {Object} center mixin
 */

var center = {
  props: {
    center: Object
  }
};
/**
 * @const {Object} radius mixin
 */

var radius = {
  props: {
    radius: Number
  }
};
/**
 * @const {Object} innerRadii mixin
 */

var innerRadii = {
  props: {
    innerRadii: Object
  }
};
/**
 * @const {Object} minimumClock mixin
 */

var minimumClock = {
  props: {
    minimumClock: {
      type: Number,
      default: 0.0
    }
  }
};
/**
 * @const {Object} maximumClock mixin
 */

var maximumClock = {
  props: {
    maximumClock: {
      type: Number,
      default: 2 * Math.PI
    }
  }
};
/**
 * @const {Object} minimumCone mixin
 */

var minimumCone = {
  props: {
    minimumCone: {
      type: Number,
      default: 0.0
    }
  }
};
/**
 * @const {Object} maximumCone mixin
 */

var maximumCone = {
  props: {
    maximumCone: {
      type: Number,
      default: Math.PI
    }
  }
};
/**
 * @const {Object} frustum mixin
 */

var frustum = {
  props: {
    frustum: Object
  }
};
/**
 * @const {Object} origin mixin
 */

var origin = {
  props: {
    origin: Object
  }
};
/**
 * @const {Object} orientation mixin
 */

var orientation = {
  props: {
    orientation: Object
  }
};
/**
 * @const {Object} polygonHierarchy mixin
 */

var polygonHierarchy = {
  props: {
    polygonHierarchy: Object | Array
  }
};

export { aaMixin, accessToken, alignedAxis, appearance, arcType, backgroundColor, backgroundPadding, blendOption, bottomRadius, center, clampAnimations, clampToGround, classificationType, clippingPlanes, clock, closeBottom, closeTop, color, colorBlendAmount, colorBlendMode, cornerType, credit, debugShowBoundingVolume, debugShowShadowVolume, debugWireframe, dimensions, disableDepthTestDistance, distanceDisplayCondition, ellipsoid, enablePickFeatures, extrudedHeight, extrudedHeightReference, eyeOffset, fileExtension, fill, fillColor, font, format, frustum, geometryInstances, getFeatureInfoFormats, granularity, height, heightReference, horizontalOrigin, id, image, incrementallyLoadTextures, innerRadii, interleave, labelStyle, layers, length, loop, material, maximumClock, maximumCone, maximumHeights, maximumLevel, maximumScale, minimumClock, minimumCone, minimumHeights, minimumLevel, minimumPixelSize, modelMatrix, numberOfVerticalLines, orientation, origin, outline, outlineColor, outlineWidth, perPositionHeight, pixelOffset, pixelOffsetScaleByDistance, pixelSize, polygonHierarchy, position, positions, radii, radius, rectangle, releaseGeometryInstances, rotation, scale, scaleByDistance, scene, semiMajorAxis, semiMinorAxis, shadows, show, showBackground, silhouetteColor, silhouetteSize, sizeInMeters, slicePartitions, slices, stRotation, stackPartitions, subdivisions, subdomains, text, tileDiscardPolicy, tileHeight, tileWidth, tilingScheme, times, token, topRadius, translucencyByDistance, uri, url, vcMixin, vertexFormat, verticalOrigin, width, zIndex };
