/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import _getIterator from '@babel/runtime-corejs2/core-js/get-iterator';
import cmp from '../virtualCmp';

var props = {
  mode: {
    type: Number,
    default: 1
  },
  show: {
    type: Boolean,
    default: true
  },
  pointColor: {
    type: String | Object | Array,
    default: 'rgb(255,229,0)'
  },
  pointPixelSize: {
    type: Number,
    default: 8
  }
};
var watch = {
  drawing: function drawing(val) {
    var nextTick, polylines, startNew, drawType, $parent, getParent, polyline, drawCmpNames, measureCmpNames, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, $node, listener;

    return _regeneratorRuntime.async(function drawing$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            nextTick = false;
            polylines = this.polylines, startNew = this.startNew, drawType = this.drawType, $parent = this.$parent, getParent = this.getParent;
            polyline = polylines[polylines.length - 1];

            if (!(!val && polyline && !polyline.positions.length)) {
              _context.next = 7;
              break;
            }

            this.polylines.pop();
            _context.next = 42;
            break;

          case 7:
            if (!val) {
              _context.next = 42;
              break;
            }

            drawCmpNames = [];
            _context.t0 = drawType;
            _context.next = _context.t0 === 'pointDrawing' ? 12 : _context.t0 === 'polylineDrawing' ? 15 : _context.t0 === 'polygonDrawing' ? 18 : 21;
            break;

          case 12:
            drawCmpNames.push('vc-handler-draw-polyline');
            drawCmpNames.push('vc-handler-draw-polygon');
            return _context.abrupt("break", 21);

          case 15:
            drawCmpNames.push('vc-handler-draw-point');
            drawCmpNames.push('vc-handler-draw-polygon');
            return _context.abrupt("break", 21);

          case 18:
            drawCmpNames.push('vc-handler-draw-polyline');
            drawCmpNames.push('vc-handler-draw-point');
            return _context.abrupt("break", 21);

          case 21:
            measureCmpNames = ['vc-measure-height', 'vc-measure-distance', 'vc-measure-area'];
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context.prev = 25;

            for (_iterator = _getIterator(getParent($parent).$slots.default || []); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              $node = _step.value;

              if ($node.componentOptions && drawCmpNames.indexOf($node.componentOptions.tag) !== -1) {
                $node.child.drawing = false;
                nextTick = true;
              }

              if ($node.componentOptions && measureCmpNames.indexOf($node.componentOptions.tag) !== -1) {
                $node.child.measuring = false;
                nextTick = true;
              }
            }

            _context.next = 33;
            break;

          case 29:
            _context.prev = 29;
            _context.t1 = _context["catch"](25);
            _didIteratorError = true;
            _iteratorError = _context.t1;

          case 33:
            _context.prev = 33;
            _context.prev = 34;

            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }

          case 36:
            _context.prev = 36;

            if (!_didIteratorError) {
              _context.next = 39;
              break;
            }

            throw _iteratorError;

          case 39:
            return _context.finish(36);

          case 40:
            return _context.finish(33);

          case 41:
            startNew();

          case 42:
            _context.t2 = nextTick;

            if (!_context.t2) {
              _context.next = 46;
              break;
            }

            _context.next = 46;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 46:
            this.viewer.canvas.setAttribute('style', val ? 'cursor: crosshair' : 'cursor: auto');
            listener = this.$listeners['activeEvt'];
            listener && this.$emit('activeEvt', {
              type: drawType,
              isActive: val
            });

          case 49:
          case "end":
            return _context.stop();
        }
      }
    }, null, this, [[25, 29, 33, 41], [34,, 36, 40]]);
  }
};
var computed = {};
var methods = {
  createCesiumObject: function createCesiumObject() {
    var viewer, handler;
    return _regeneratorRuntime.async(function createCesiumObject$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            viewer = this.viewer;
            handler = new Cesium.ScreenSpaceEventHandler(viewer.canvas);
            handler.setInputAction(this.LEFT_CLICK, Cesium.ScreenSpaceEventType.LEFT_CLICK);
            handler.setInputAction(this.MOUSE_MOVE, Cesium.ScreenSpaceEventType.MOUSE_MOVE);
            handler.setInputAction(this.RIGHT_CLICK, Cesium.ScreenSpaceEventType.RIGHT_CLICK);
            this.handler = handler;
            return _context2.abrupt("return", this.polylines);

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, null, this);
  },
  mount: function mount() {
    return _regeneratorRuntime.async(function mount$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt("return", true);

          case 1:
          case "end":
            return _context3.stop();
        }
      }
    });
  },
  unload: function unload() {
    return _regeneratorRuntime.async(function unload$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            return _context4.abrupt("return", true);

          case 1:
          case "end":
            return _context4.stop();
        }
      }
    });
  },
  LEFT_CLICK: function LEFT_CLICK(movement) {
    if (!this.drawing) {
      return;
    }

    var Cesium = this.Cesium,
        viewer = this.viewer,
        polylines = this.polylines,
        onDrawingEvt = this.onDrawingEvt;
    var cartesian = viewer.scene.pickPosition(movement.position);

    if (!Cesium.defined(cartesian)) {
      return;
    }

    var nIndex = polylines.length - 1;
    var polyline = polylines[nIndex];
    polyline.positions.push(cartesian);
    onDrawingEvt(polyline, nIndex);
  },
  MOUSE_MOVE: function MOUSE_MOVE(movement) {
    var Cesium, viewer, polylines, onDrawingEvt, drawType, nIndex, polyline, cartesian, listener;
    return _regeneratorRuntime.async(function MOUSE_MOVE$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            if (this.drawing) {
              _context5.next = 2;
              break;
            }

            return _context5.abrupt("return");

          case 2:
            Cesium = this.Cesium, viewer = this.viewer, polylines = this.polylines, onDrawingEvt = this.onDrawingEvt, drawType = this.drawType;

            if (polylines.length) {
              _context5.next = 5;
              break;
            }

            return _context5.abrupt("return");

          case 5:
            nIndex = polylines.length - 1;
            polyline = polylines[nIndex];

            if (polyline.positions.length) {
              _context5.next = 9;
              break;
            }

            return _context5.abrupt("return");

          case 9:
            cartesian = viewer.scene.pickPosition(movement.endPosition);

            if (Cesium.defined(cartesian)) {
              _context5.next = 12;
              break;
            }

            return _context5.abrupt("return");

          case 12:
            if (polyline.positions.length >= 2) {
              polyline.positions.pop();
            }

            polyline.positions.push(cartesian);
            listener = this.$listeners['movingEvt'];
            listener && this.$emit('movingEvt', movement.endPosition, drawType);
            onDrawingEvt(polyline, nIndex);

          case 17:
          case "end":
            return _context5.stop();
        }
      }
    }, null, this);
  },
  RIGHT_CLICK: function RIGHT_CLICK(movement) {
    var viewer, polylines, mode, startNew, onDrawingEvt, nIndex, polyline, cartesian;
    return _regeneratorRuntime.async(function RIGHT_CLICK$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            if (this.drawing) {
              _context6.next = 2;
              break;
            }

            return _context6.abrupt("return");

          case 2:
            viewer = this.viewer, polylines = this.polylines, mode = this.mode, startNew = this.startNew, onDrawingEvt = this.onDrawingEvt;

            if (polylines.length) {
              _context6.next = 5;
              break;
            }

            return _context6.abrupt("return");

          case 5:
            nIndex = polylines.length - 1;
            polyline = polylines[nIndex];

            if (!(polyline.positions.length === 0)) {
              _context6.next = 9;
              break;
            }

            return _context6.abrupt("return");

          case 9:
            cartesian = viewer.scene.pickPosition(movement.position);

            if (Cesium.defined(cartesian)) {
              _context6.next = 12;
              break;
            }

            return _context6.abrupt("return");

          case 12:
            polyline.positions.pop();

            if (mode === 0) {
              startNew();
            } else {
              this.drawing = false;
            }

            _context6.next = 16;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 16:
            onDrawingEvt(polyline, nIndex, true);

          case 17:
          case "end":
            return _context6.stop();
        }
      }
    }, null, this);
  },
  startNew: function startNew() {
    var polylines = this.polylines;
    var poyline = {
      positions: []
    };
    Cesium.defined(polylines) && polylines.push(poyline);
  },
  clear: function clear() {
    this.polylines = [];
    this.drawing = false;
  },
  onDrawingEvt: function onDrawingEvt(polyline, index) {
    var flag = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    this.index = index;

    if (!this.depthTest) {
      var rs = Cesium.RenderState.fromCache({
        depthMask: true,
        depthTest: {
          enabled: false
        }
      });
      this.$refs.pointCollection && (this.$refs.pointCollection.cesiumObject._rsOpaque = rs);

      if (Cesium.SuperMapImageryProvider) {
        this.$refs.polylineCollection && (this.$refs.polylineCollection.cesiumObject._opaqueRS = rs);
      } else {
        this.$refs.polylineCollection && (this.$refs.polylineCollection.cesiumObject._opaqueRS.depthTest.enabled = false);
      }
    }

    var listener = this.$listeners['drawEvt'];
    listener && this.$emit('drawEvt', {
      polyline: polyline,
      type: this.drawType,
      finished: flag
    });
  }
};
var mixinDraw = {
  mixins: [cmp],
  props: props,
  watch: watch,
  computed: computed,
  methods: methods,
  destroyed: function destroyed() {
    var handler = this.handler;

    if (handler) {
      handler.removeInputAction(Cesium.ScreenSpaceEventType.LEFT_CLICK);
      handler.removeInputAction(Cesium.ScreenSpaceEventType.MOUSE_MOVE);
      handler.removeInputAction(Cesium.ScreenSpaceEventType.RIGHT_CLICK);
    }
  }
};

export default mixinDraw;
