/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import _Object$defineProperties from '@babel/runtime-corejs2/core-js/object/define-properties';
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import cmp from '../mixins/virtualCmp';
import mergeDescriptors from '../utils/mergeDescriptors';

var script = {
  name: 'vc-collection-stage-process-post',
  mixins: [cmp],
  props: {
    stages: {
      type: Array,
      default: function _default() {
        return [];
      }
    }
  },
  methods: {
    createCesiumObject: function createCesiumObject() {
      var transformProps, stages, postProcessStageCollection;
      return _regeneratorRuntime.async(function createCesiumObject$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              transformProps = this.transformProps, stages = this.stages;
              postProcessStageCollection = new Cesium.PostProcessStageCollection();
              stages.forEach(function (stage) {
                var stageOptions = transformProps(stage);
                postProcessStageCollection.add(new Cesium.PostProcessStage(stageOptions));
              });
              return _context.abrupt("return", postProcessStageCollection);

            case 4:
            case "end":
              return _context.stop();
          }
        }
      }, null, this);
    },
    mount: function mount() {
      var postProcessStages, viewer;
      return _regeneratorRuntime.async(function mount$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              postProcessStages = this.postProcessStages, viewer = this.viewer;
              viewer.scene.postProcessStages = postProcessStages;
              return _context2.abrupt("return", true);

            case 3:
            case "end":
              return _context2.stop();
          }
        }
      }, null, this);
    },
    unmount: function unmount() {
      var postProcessStages;
      return _regeneratorRuntime.async(function unmount$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              postProcessStages = this.postProcessStages;
              postProcessStages.destroy();
              return _context3.abrupt("return", true);

            case 3:
            case "end":
              return _context3.stop();
          }
        }
      }, null, this);
    },
    getServices: function getServices() {
      var vm = this;
      return mergeDescriptors(cmp.methods.getServices.call(this), {
        get postProcessStages() {
          return vm.postProcessStages;
        },

        get postProcessStagesContainer() {
          return vm;
        }

      });
    }
  },
  created: function created() {
    var _this = this;

    _Object$defineProperties(this, {
      postProcessStages: {
        enumerable: true,
        get: function get() {
          return _this.cesiumObject;
        }
      }
    });
  },
  stubVNode: {
    attrs: function attrs() {
      return {
        class: this.$options.name
      };
    }
  }
};

/* script */
var __vue_script__ = script;
/* template */

/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* functional template */

var __vue_is_functional_template__ = undefined;
/* component normalizer */

function __vue_normalize__(template, style, script, scope, functional, moduleIdentifier, createInjector, createInjectorSSR) {
  var component = (typeof script === 'function' ? script.options : script) || {}; // For security concerns, we use only base name in production mode.

  component.__file = "PostProcessStageCollection.vue";

  if (!component.render) {
    component.render = template.render;
    component.staticRenderFns = template.staticRenderFns;
    component._compiled = true;
    if (functional) component.functional = true;
  }

  component._scopeId = scope;

  return component;
}
/* style inject */

/* style inject SSR */


var PostProcessStageCollection = __vue_normalize__({}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__);

function plugin(Vue) {

  if (plugin.installed) {
    return;
  }

  plugin.installed = true;
  Vue.component(PostProcessStageCollection.name, PostProcessStageCollection);
}

export default plugin;
export { PostProcessStageCollection, plugin as install };
