/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import mixinDraw from '../../mixins/tool/mixinDraw';
import { makeMaterial } from '../../utils/cesiumHelpers';

//
var script = {
  name: 'vc-handler-draw-polyline',
  mixins: [mixinDraw],
  data: function data() {
    return {
      drawType: 'polylineDrawing',
      drawing: false,
      polylines: [],
      nowaiting: true
    };
  },
  props: {
    depthTest: {
      type: Boolean,
      default: false
    },
    polylineMaterial: {
      type: Object,
      default: function _default() {
        return {
          fabric: {
            type: 'Color',
            uniforms: {
              color: '#51ff00'
            }
          }
        };
      }
    },
    polylineWidth: {
      type: Number,
      default: 2
    },
    clampToGround: {
      type: Boolean,
      default: false
    }
  },
  methods: {
    makeAppearance: function makeAppearance(val) {
      return new Cesium.PolylineMaterialAppearance({
        material: makeMaterial.call(this, val)
      });
    }
  }
};

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('i', {
    class: _vm.$options.name,
    staticStyle: {
      "display": "none !important"
    }
  }, [_c('vc-collection-primitive', {
    attrs: {
      "show": _vm.show
    }
  }, [_vm.clampToGround ? _c('vc-collection-primitive', {
    ref: "groundPolylineCollection"
  }, [_vm._l(_vm.polylines, function (polyline, index) {
    return [polyline.positions.length > 1 ? _c('vc-primitive-polyline-ground', {
      key: index,
      attrs: {
        "appearance": _vm.makeAppearance(_vm.polylineMaterial),
        "asynchronous": false
      }
    }, [_c('vc-instance-geometry', [_c('vc-geometry-polyline-ground', {
      attrs: {
        "positions": polyline.positions,
        "width": _vm.polylineWidth
      }
    })], 1)], 1) : _vm._e()];
  })], 2) : _c('vc-collection-primitive-polyline', {
    ref: "polylineCollection"
  }, _vm._l(_vm.polylines, function (polyline, index) {
    return _c('vc-primitive-polyline', {
      key: index,
      attrs: {
        "material": _vm.polylineMaterial,
        "positions": polyline.positions,
        "width": _vm.polylineWidth
      }
    });
  }), 1), _vm._v(" "), _c('vc-collection-primitive-point', {
    ref: "pointCollection"
  }, [_vm._l(_vm.polylines, function (polyline, index) {
    return [_vm._l(polyline.positions, function (position, subIndex) {
      return [_c('vc-primitive-point', {
        key: 'point' + index + 'position' + subIndex,
        attrs: {
          "color": _vm.pointColor,
          "pixelSize": _vm.pointPixelSize,
          "position": position
        }
      })];
    })];
  })], 2)], 1)], 1);
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* functional template */

var __vue_is_functional_template__ = false;
/* component normalizer */

function __vue_normalize__(template, style, script, scope, functional, moduleIdentifier, createInjector, createInjectorSSR) {
  var component = (typeof script === 'function' ? script.options : script) || {}; // For security concerns, we use only base name in production mode.

  component.__file = "VcDrawHandlerPolyline.vue";

  if (!component.render) {
    component.render = template.render;
    component.staticRenderFns = template.staticRenderFns;
    component._compiled = true;
    if (functional) component.functional = true;
  }

  component._scopeId = scope;

  return component;
}
/* style inject */

/* style inject SSR */


var VcDrawHandlerPolyline = __vue_normalize__({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__);

function plugin(Vue) {

  if (plugin.installed) {
    return;
  }

  plugin.installed = true;
  Vue.component(VcDrawHandlerPolyline.name, VcDrawHandlerPolyline);
}

export default plugin;
export { VcDrawHandlerPolyline, plugin as install };
