class ColorTable {
  constructor () {
    const { AssociativeArray } = Cesium
    this._dictTable = new AssociativeArray()
    this._imageBuffer = new Uint8Array(262144)
    this._sortKey = []
  }

  destroy () {
    this._dictTable.removeAll()
    this._imageBuffer = null
    this._sortKey = null
  }

  getItem (e) {
    if (e > this._dictTable.values.length - 1) {
      return null
    }
    let lastKey
    let value
    let index = e
    for (let key in this._dictTable._hash) {
      if (!(index > 0)) {
        if (!Cesium.defined(0)) { break }
        lastKey = key
        value = this._dictTable.get(key)
        return {
          altitude: lastKey,
          color: value
        }
      }
      index--
    }
    return null
  }

  count () {
    return this._dictTable.values.length
  }

  insert (key, color) {
    const { Color, defined } = Cesium
    let keyValuePair = this._dictTable.get(key)
    if (defined(keyValuePair)) {
      let newColor = Color.clone(color)
      this._dictTable.set(key, newColor)
    }
  }

  remove (key) {
    return this._dictTable.remove(key)
  }

  clear () {
    this._dictTable.removeAll()
  }

  generateBuffer () {
    const { clone } = Cesium
    const colors = Object.keys(this._dictTable._hash).reduce((pre, key) => {
      const value = this._dictTable.get(key)
      return pre.concat({ value: parseFloat(key), color: value })
    }, [])
    if (colors.length >= 2) {
      for (let i = 0; i < colors.length - 1; i++) {
        let colorI = colors[i]
        for (let s = i + 1; s < colors.length; s++) {
          let colorS = colors[s]
          if (colorI.value > colorS.value) {
            let colorINew = clone(colorI, false)
            colors[i] = clone(colorS, false)
            colors[s] = colorINew
            colorI = colors[i]
          }
        }
      }
      let firstVal = colors[0].value
      let lastVal = colors[colors.length - 1].value
      let divVal = lastVal - firstVal
      let c = document.createElement('canvas')
      c.width = 16384
      c.height = 1
      let ctx = c.getContext('2d')
      ctx.beginPath()
      let linear = ctx.createLinearGradient(0, 0, 16384, 0)
      for (let i = 0; i < colors.length; i++) {
        linear.addColorStop((colors[i].value - firstVal) / divVal, colors[i].color.toCssColorString())
      }
      ctx.fillStyle = linear
      ctx.rect(0, 0, c.width, c.height)
      ctx.fill()
      let imgData = ctx.getImageData(0, 0, c.width, 1).data
      for (let s = 0; s < 16; s++) {
        for (let i = 0; i < 4096; i++) {
          this._imageBuffer[i + 1024 * s * 4 * 4] = imgData[i + 1024 * s * 4]
          this._imageBuffer[i + 1024 * s * 4 * 4 + 4096] = imgData[i + 1024 * s * 4]
          this._imageBuffer[i + 1024 * s * 4 * 4 + 8192] = imgData[i + 1024 * s * 4]
          this._imageBuffer[i + 1024 * s * 4 * 4 + 12288] = imgData[i + 1024 * s * 4]
        }
      }
      for (let r in this._dictTable._hash) {
        this._sortKey.push(parseFloat(r))
      }
      this._sortKey.sort(sortNumber)
      return c
    }
  }
}

function sortNumber (a, b) {
  return a - b
}

export default ColorTable
